;;;; -*- Mode: lisp; indent-tabs-mode: nil -*-
;;;
;;; speed.lisp --- Speed measurement for Babel.
;;;
;;; Copyright (C) 2007-2008, Luis Oliveira  <loliveira@common-lisp.net>
;;;
;;; Permission is hereby granted, free of charge, to any person
;;; obtaining a copy of this software and associated documentation
;;; files (the "Software"), to deal in the Software without
;;; restriction, including without limitation the rights to use, copy,
;;; modify, merge, publish, distribute, sublicense, and/or sell copies
;;; of the Software, and to permit persons to whom the Software is
;;; furnished to do so, subject to the following conditions:
;;;
;;; The above copyright notice and this permission notice shall be
;;; included in all copies or substantial portions of the Software.
;;;
;;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
;;; EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
;;; MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
;;; NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
;;; HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
;;; WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
;;; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
;;; DEALINGS IN THE SOFTWARE.

(in-package #:cl-user)
(defpackage #:babel-speed
  (:use #:common-lisp #:babel #:babel-encodings))
(in-package #:babel-speed)

(defun run-speed ()
  "This is the top-level function, called from the speed-op ASDF operation."
  (format t "~&Speed measurement of decoding UTF8 by Babel.~%")
  (format t "Decode the test file...")
  (format t " Times: ~s~%" (decode-file))
  (loop for length in '(0 2 5 10 50 100)
        for repeats in '(10000000 10000000 10000000 10000000 5000000 1000000)
        do (format t "Decode ~d bytes..." length)
           (format t " Times: ~s~%" (decode-array length repeats))))

(defun test-file (name type)
  "Copied from test suite.  Get the pathname for the test file
   designated by NAME and TYPE."
  (let ((sys-pn (truename
                 (asdf:system-definition-pathname
                  (asdf:find-system 'babel-tests)))))
    (make-pathname :name name :type type
                   :directory (append (pathname-directory sys-pn)
                                      '("tests"))
                   :defaults sys-pn)))

;;; Note: the test file contains 14052 octets
(defun read-test-file (name type)
  "Copied from test suite.  Read in the test file designated by NAME and TYPE.
   Return the contents as an array of octets (unsigned bytes)."
  (with-open-file (in (test-file name type) :element-type '(unsigned-byte 8))
    (let* ((data (loop for byte = (read-byte in nil nil)
                       until (null byte) collect byte)))
      (make-array (length data) :element-type '(unsigned-byte 8)
                  :initial-contents data))))

(defun decode-octets (utf8-octets iter)
  "UTF8-OCTETS is an array of octets.  Five times, decode the array ITER
   times.  Return a list of five elements.  Each element is the number of
   microseconds to decode the array.  It's done this way so that a
   person can discard outliers."
  (let ((times nil))
    (dotimes (i 5)
      (let ((start-time (quux:log-get-time)))
        (dotimes (j iter)
          (octets-to-string utf8-octets #|| :encoding :utf-8 ||#))
        (let ((end-time (quux:log-get-time)))
          (push (/ (float (- end-time start-time)) 
                   (float iter))
                times))))
      times))

(defun decode-file ()
  "Return the 5-list characterizing the time it takes to decode the
   contents of the test file."
  (decode-octets (read-test-file "utf-32" "txt-utf8") 10000))

(defun decode-array (length &optional (repeats 10000000))
  "Return the 5-list characterizing the time it takes to decode a
   byte array the given LENGTH."
  (decode-octets (make-array length :element-type '(unsigned-byte 8) :initial-element 85)
                 repeats))
